#
# $Id: invoice.d,v 1.25 2008/01/15 18:10:40 dtynan Exp $
#
# Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
# Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
# This is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2, or (at your
# option) any later version.
#
# It is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
# License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this product; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
# USA.
#
# THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
# RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# ABSTRACT
# This file contains the tables needed to maintain invoice information
# sent to customers.
#
# The invoice table has the following field:
#	id		A unique invoice number.
#	idate		Invoice date / Tax Point
#	ddate		Date invoice moves to next state
#	odate		Date when order was first entered
#	customer_id	Reference to the Customer
#	salesorder_id	Reference to the Sales Order
#	cninv_id	Non-zero if a CREDIT NOTE (references invoice)
#	currency_id	Reference to the invoice currency
#	total		Invoice GROSS total (in INVOICE currency)
#	htotal		Invoice GROSS total (in LOCAL currency)
#	vatttl		Invoice VAT amount (in invoice currency)
#	shipping	Invoice shipping charges (in invoice currency)
#	yref		"Your Reference"
#	descr		One line description of invoice
#	state		Current state (see table)
#
# For each invoice there are one or more invoice_items which represent
# line-items on the invoice.  The following is the field description:
#	invoice_id	Invoice number (see above)
#	item_id		Line-item number (1 through NNN)
#	nomacct_id	Nominal account code to credit
#	descr		Line-item description
#	rdate		Date of transaction (for revenue recognition)
#	runlen		Duration of transaction ("" "")
#	vcode		VAT Code (see vat.d for further information)
#	quant		Quantity
#	unitc		Unit cost in Euro, dollars, whatever
#	disc		Discount (percentage)
#
# $Log: invoice.d,v $
# Revision 1.25  2008/01/15 18:10:40  dtynan
# Changed to use new company name and copyright, also fixed a few old
# files with the wrong license.
#
# Revision 1.24  2007/10/29 18:15:42  dtynan
# Added comments for additional fields in invoice tables.
#
# Revision 1.23  2007/10/29 17:13:37  dtynan
# Various database changes to support hourly billing.
#
# Revision 1.22  2006/12/18 08:45:13  dtynan
# Added a currency field to the invoice.
#
# Revision 1.21  2006/10/01 10:14:50  dtynan
# Created new mechanism for running overnight tasks.
#
# Revision 1.20  2006/08/25 11:27:12  dtynan
# Lots of changes as per the end-of-year.
#
# Revision 1.19  2006/07/31 09:07:54  dtynan
# Renamed all primary keys from xxx_id to id (where xxx is the table
# name).  This is both convenient and supports the Ruby/Rails
# methodology.
#
# Revision 1.18  2005/12/18 15:59:18  dtynan
# Removed hard-coded paths from Makefile and added date and runlength
# fields to invoice item for revenue recognition and for producing
# credit notes.
#
# Revision 1.17  2005/10/17 13:27:43  dtynan
# First pass at multicurrency changes.
#
# Revision 1.16  2005/09/21 18:02:29  dtynan
# Major changes to deal with data import/export.
#
# Revision 1.15  2005/08/12 18:44:53  dtynan
# Added a new comment in each file to show whether the table belongs in
# the main or the per-company databases.
#
# Revision 1.14  2005/03/22 09:41:11  dtynan
# Extensive upgrade to make sure the copyright block was
# consistent and GPL'ed.
#
# Revision 1.13  2005/03/11 13:15:44  dtynan
# Added new sales order table entry (will require drastic changes later
# on).  For now, just track the sales order with the invoice.
#
# Revision 1.12  2005/01/13 17:44:26  dtynan
# Added new program to post a customer payment to the database.  Also did cleanups
# of functions across the board.
#
# Revision 1.11  2004/12/05 22:58:57  dtynan
# Added a 'folio' field to the journal and totals to the invoice.
#
# Revision 1.10  2004/10/08 10:39:50  dtynan
# Added field definitions.
#
# Revision 1.9  2004/08/04 13:47:42  dtynan
# Added sort option.
#
# Revision 1.8  2004/07/16 19:19:47  dtynan
# Added nominal account number to each invoice record.
#
# Revision 1.7  2004/05/16 21:42:30  dtynan
# Added additional fields for new invoice formats.
#
# Revision 1.6  2004/05/12 15:21:09  dtynan
# Massive check-in of various changes, mostly around DBOW and directory
# reorganization (moved to /u/beanie).
#
# Revision 1.5  2004/01/05 15:36:36  dtynan
# Added hacked-function to find overdue invoices.
#
# Revision 1.4  2003/11/08 16:34:52  dtynan
# Fixed some bugs with hard-coded functions.
#
# Revision 1.3  2003/11/08 15:09:20  dtynan
# New database table for managing and maintaining sites, also changed
# some stuff in the company table and the invoice table.
#
# Revision 1.2  2003/11/07 20:15:58  dtynan
# Changed the currency 'rate' value to a double and hard-coded a C function
# for updating the database (this support is missing from DBOW).
#
# Revision 1.1  2003/10/31 14:06:31  dtynan
# First pass at DBOW files.
#
# COMPANYDB
#
%table	invoice	{
	id mediumint(7) not null auto_increment primary key,
	idate date not null,
	ddate date not null,
	odate date not null,
	customer_id mediumint(7) not null,
	salesorder_id mediumint(7) not null,
	cninv_id mediumint(7),
	currency_id mediumint(7) not null,
	total real,
	htotal real,
	vatttl real,
	shipping real,
	yref varchar(254),
	descr varchar(254),
	state tinyint(3)
%}

%insert invoice
%search invoice id
%search invoice customer_id
%search invoice state
%dump invoice

%table	invoice_item {
	invoice_id mediumint(7) NOT NULL primary key,
	item_id mediumint(7) NOT NULL primary key,
	nomacct_id mediumint(7),
	descr varchar(254) NOT NULL,
	rdate date NOT NULL,
	runlen smallint(5),
	vcode tinyint,
	quant real,
	unitc real,
	disc real
%}

%search invoice_item invoice_id
%dump invoice_item

%type C proto {
/*
 * Values for the 'state' field in the invoice record.
 */
#define INVOICE_STATE_NEW	0		/* Created but not stuffed */
#define INVOICE_STATE_PAID	1		/* Invoice is paid */
#define INVOICE_STATE_DEAD	2		/* Invoice was cancelled */
#define INVOICE_STATE_SENT	3		/* Invoice sent to customer */
#define INVOICE_STATE_LATE1	4		/* Invoice late (> 30 days) */
#define INVOICE_STATE_LATE2	5		/* Invoice late (> 45 days) */
#define INVOICE_STATE_LATE3	6		/* Invoice late (> 60 days) */
#define INVOICE_STATE_LATE4	7		/* Invoice late (> 90 days) */
#define INVOICE_STATE_LATE5	8		/* Invoice late (> 120 days) */
%}

%type C emit {
/*
 *
 */
struct db_invoice *
findoverdueinvoice(dbow_conn *c)
{
	return(db_runinvoicequery(c,
		"SELECT * FROM invoice WHERE state>2 AND ddate<SYSDATE() ORDER BY id"));
}

/*
 *
 */
struct db_invoice_item *
findinvoiceitem(dbow_conn *c, int invno, int itemno)
{
	if (dbow_query(c, "SELECT * FROM invoice_item WHERE invoice_id = $c AND item_id = $c", invno, itemno) < 0)
		return(NULL);
	return(db_findinvoice_itemnext(c, NULL));
}
%}
