#ident "Kalopa: $Id: datestuff.c,v 1.10 2008/01/15 18:10:41 dtynan Exp $"

/*
 * $Id: datestuff.c,v 1.10 2008/01/15 18:10:41 dtynan Exp $
 *
 * Copyright (c) 2003, Kalopa Media Limited.  All rights reserved.
 * Copyright (c) 2005, Kalopa Research Limited.  All rights reserved.
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * It is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this product; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 * THIS SOFTWARE IS PROVIDED BY KALOPA RESEARCH LIMITED "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL KALOPA
 * RESEARCH LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ABSTRACT
 *
 * $Log: datestuff.c,v $
 * Revision 1.10  2008/01/15 18:10:41  dtynan
 * Changed to use new company name and copyright, also fixed a few old
 * files with the wrong license.
 *
 * Revision 1.9  2005/04/11 17:47:20  dtynan
 * Added a new function to print a date string dd-Mmm-YYYY.
 *
 * Revision 1.8  2005/03/22 09:41:16  dtynan
 * Extensive upgrade to make sure the copyright block was
 * consistent and GPL'ed.
 *
 * Revision 1.7  2005/03/08 16:36:55  dtynan
 * Added code to handle parsing the time as well as the date.
 *
 * Revision 1.6  2004/12/09 19:19:13  dtynan
 * Added extra functions to produce date strings.
 *
 * Revision 1.5  2004/08/26 17:50:09  dtynan
 * Added code to parse 8-digit dates as well.
 *
 * Revision 1.4  2004/08/26 17:29:04  dtynan
 * New function to parse a date.
 *
 * Revision 1.3  2004/05/12 15:21:14  dtynan
 * Massive check-in of various changes, mostly around DBOW and directory
 * reorganization (moved to /u/beanie).
 *
 * Revision 1.2  2003/11/08 16:01:06  dtynan
 * Improved handling of cardinal numbers and fixed data types in year-end.
 *
 * Revision 1.1  2003/11/08 14:53:48  dtynan
 * Added functions to getyend.c to determine quarter-ends and where we are
 * in the quarter.  Also added new functions for date manipulation, roff
 * processing and initialization.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "beanie.h"

static	const	char	*lmonths[12] = {
	"January","February","March","April","May","June","July",
	"August","September","October","November","December"
};

static	const	char	*smonths[12] = {
	"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
};

/*
 *
 */
char *
prdate(int day, int mon, int year)
{
	char *cp;
	static char buffer[512];

	cp = buffer;
	if (mon > 0 && mon <= 12) {
		strcpy(cp, lmonths[mon-1]);
		cp = strchr(cp, '\0');
	}
	if (day > 0) {
		if (cp > buffer)
			*cp++ = ' ';
		sprintf(cp, "%d", day);
		cp = strchr(cp, '\0');
		switch ((day < 11 || day > 13) ? day % 10 : 0) {
		case 1:
			*cp++ = 's';
			*cp++ = 't';
			break;
		case 2:
			*cp++ = 'n';
			*cp++ = 'd';
			break;
		case 3:
			*cp++ = 'r';
			*cp++ = 'd';
			break;
		default:
			*cp++ = 't';
			*cp++ = 'h';
			break;
		}
		*cp = '\0';
	}
	if (year > 0) {
		if (cp > buffer) {
			*cp++ = ',';
			*cp++ = ' ';
		}
		if (year < 100)
			year += 1900;
		sprintf(cp, "%04d", year);
	}
	return(buffer);
}

/*
 *
 */
char *
prtmptr(struct tm *tmp)
{
	return(prdate(tmp->tm_mday, tmp->tm_mon + 1, tmp->tm_year + 1900));
}

/*
 *
 */
char *
prabbrev(time_t *when)
{
	struct tm *tmp;
	static char buffer[512];

	if ((tmp = localtime(when)) == NULL)
		return(NULL);
	sprintf(buffer, "%2d-%s-%04d", tmp->tm_mday, smonths[tmp->tm_mon],
						tmp->tm_year + 1900);
	return(buffer);
}

/*
 *
 */
char *
prtime(time_t *when)
{
	struct tm *tmp;

	if ((tmp = localtime(when)) != NULL)
		return(prtmptr(tmp));
	return(NULL);
}

/*
 * Parse a date string into a time_t based on some known heuristics.
 * The valid string types can be as follows:
 *	YYYYMMDDhhmmss
 *	YYYYMMDD
 *	YYMMDD
 *	YYYY-MM-DD
 *	DD/MM/YYYY
 *	DD/MM/YY
 */
int
parsedate(char *dstr, time_t *nowp)
{
	int n, y;
	struct tm *tp, tmstr;
	char *cp;

	*nowp = 0;
	tp = &tmstr;
	tp->tm_isdst = -1;
	tp->tm_hour = 18;
	tp->tm_min = tp->tm_sec = 0;
	if ((cp = strchr(dstr, '/')) != NULL) {
		*cp++ = '\0';
		if ((tp->tm_mday = atoi(dstr)) < 1 || tp->tm_mday > 31)
			return(-1);
		dstr = cp;
		if ((cp = strchr(dstr, '/')) == NULL)
			return(-1);
		*cp++ = '\0';
		if ((tp->tm_mon = atoi(dstr) - 1) < 0 || tp->tm_mon > 11)
			return(-1);
		/*
		 * Do usual millenial assumptions for sloppy typists.
		 */
		if ((y = atoi(cp)) < 100)
			y += (y < 70) ? 2000 : 1900;
	} else if ((cp = strchr(dstr, '-')) != NULL) {
		*cp++ = '\0';
		if ((y = atoi(dstr)) < 100)
			y += (y < 70) ? 2000 : 1900;
		dstr = cp;
		if ((cp = strchr(dstr, '-')) == NULL)
			return(-1);
		*cp++ = '\0';
		if ((tp->tm_mon = atoi(dstr) - 1) < 0 || tp->tm_mon > 11)
			return(-1);
		if ((tp->tm_mday = atoi(cp)) < 1 || tp->tm_mday > 31)
			return(-1);
	} else {
		if ((n = strlen(dstr)) != 6 && n != 8 && n != 14)
			return(-1);
		if (n == 14) {
			/*
			 * The date has a time attached.  Deal with it.
			 */
			if ((y = atoi(dstr + 8)) < 0)
				return(-1);
			if ((tp->tm_sec = y % 100) < 0 || tp->tm_sec > 59)
				return(-1);
			y /= 100;
			if ((tp->tm_min = y % 100) < 0 || tp->tm_min > 59)
				return(-1);
			y /= 100;
			if ((tp->tm_hour = y % 100) < 0 || tp->tm_hour > 23)
				return(-1);
			dstr[8] = '\0';
		}
		if ((y = atoi(dstr)) < 1)
			return(-1);
		if (n == 6) {
			/*
			 * The century has been omitted.  Tut tut!
			 * If the "year" is less than 70 then we
			 * presume it's the 21st century otherwise
			 * it's the 20th century.
			 */
			y += (y < 700000) ? 20000000 : 19000000;
		}
		/*
		 * Nothing happened before 1960 and nothing will work
		 * at the end of this millenium.  I'll be dead anyway.
		 */
		if (y < 19600000 || y > 29990000)
			return(-1);
		if ((tp->tm_mday = y % 100) < 1 || tp->tm_mday > 31)
			return(-1);
		y /= 100;
		if ((tp->tm_mon = y % 100 - 1) < 0 || tp->tm_mon > 11)
			return(-1);
		y /= 100;
	}
	tp->tm_year = y - 1900;
	tp->tm_isdst = -1;
	*nowp = mktime(tp);
	return(0);
}
